;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; Rutinas de envio de datos a 57600 bps por el puerto del usuario de C64, y de impresion en pantalla caracter a caracter
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; Ultima modificacion: 14-Feb-2016
;
; Versiones / correcciones
; ========================
; 30-11-2016	Primera version, partiendo desde backtest5.asm.
;========================================================================================

; Variables

	TXBYTE = $C015		; Variable usada para contener el byte a enviar
	TXBUFFERCNT = $C016	; Contador de elementos del buffer de transmision
	PRBUFFERCNT = $C017	; Contador de elementos del buffer de impresion
	TXINDEXOUT = $C018	; Indice al proximo elemento a enviar del buffer de transmision
	TXINDEXIN = $C019	; Indice al proximo elemento libre del buffer de transmision
	PRINDEXOUT = $C01A	; Indice al proximo elemento a imprimir del buffer de impresion
	PRINDEXIN = $C01B	; Indice al proximo elemento libre del buffer de impresion
	VARPTR = $C01C		; Puntero a la tabla de variables
	CONTROLBYTE = $C01E	; Bit 0: 0 = No interfiere con el cursor; 1 = Fuerza visibilidad del cursor	49182
	CHARCNT = $C01F		; Contador de caracteres
	PRBYTE = $C020		; Variable usada para contener el byte a imprimir
	RXBYTE = $C021		; Variable usada para contener el byte recibido
	VERSION = $C022		; Espacio de 6 bytes donde se almacenara el modelo y version del sintetizador	49186

; Constantes

	SIDREG = $D400		; Comienzo de los registros del SID
	VICREG = $D000		; Comienzo de los registros del VIC II
	PLOT   = $FFF0		; Llamada a la rutina PLOT de la ROM Kernal
	CHROUT = $FFD2		; Llamada a la rutina CHROUT de la ROM Kernal
	STROUT = $AB1E		; Llamada a la rutina STROUT de la ROM BASIC
	COLORMEM = $D800	; Comienzo de la memoria de color de caracteres
	VARTAB = $002D		; Puntero al comienzo de la tabla de variables simples
	ARYTAB = $002F		; Puntero al comienzo de la tabla de arreglos/matrices

	!to "txdrv5", cbm
;	!sl "labels.txt"

	*= $C000

;///////////////////////////////////////////////////////////////////////////////////
; INICIO (2083)
;///////////////////////////////////////////////////////////////////////////////////

Start
	JMP	SendText	; Envia por RS232 el texto contenido en la variable ZS$			49152
	JMP	PrintText	; Imprime caracter a caracter el texto contenido en la variable ZP$	49155
	JMP	WaitSend	; Espera a que se haya enviado completo el buffer de transmision	49158
	JMP	WaitPrint	; Espera a que se haya impreso completo el buffer de impresion		49161
	JMP	InitIrq		; Inicializa las interrupciones en InitIrq				49164
	JMP	RestoreIrq	; Restaura las interrupciones de la ROM					49167
	JMP	ChkVersion	; Detecta si hay un sintetizador y chequea modelo y version		49170

	*= $C028

;///////////////////////////////////////////////////////////////////////////////////
; SUBRUTINAS
;///////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////////////////////////////////////////////////
; Subrutina SendByte, recibe en A el byte a enviar

SendByte
	STA	TXBYTE		; Guarda el contenido de A en TXBYTE
	LDA	$DD00		; Hace una copia de PRA (puerto A, CIA2)
	AND	#$FB		; Coloca un 0 en TX (bit de start)
	TAX			; y hace una copia en el registro X
SendStart
	STA	$DD00		; *** Comienzo del bit de start ***
;---------------------------------------------------------------------------------------
CalcB0
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 0 de TXBYTE
	BCS	Bit0Is1		; 2 Si CARRY=1 sigue en Bit0Is1
Bit0Is0
	JMP	SendB0		; 3 Si CARRY=0, sigue en SendB0
Bit0Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB0
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 0 ***
;---------------------------------------------------------------------------------------
CalcB1
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 1 de TXBYTE
	BCS	Bit1Is1		; 2 Si CARRY=1 sigue en Bit1Is1
Bit1Is0
	JMP	SendB1		; 3 Si CARRY=0, sigue en SendB1
Bit1Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB1
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 1 ***
;---------------------------------------------------------------------------------------
CalcB2
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 2 de TXBYTE
	BCS	Bit2Is1		; 2 Si CARRY=1 sigue en Bit2Is1
Bit2Is0
	JMP	SendB2		; 3 Si CARRY=0, sigue en SendB2
Bit2Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB2
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 2 ***
;---------------------------------------------------------------------------------------
CalcB3
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 3 de TXBYTE
	BCS	Bit3Is1		; 2 Si CARRY=1 sigue en Bit3Is1
Bit3Is0
	JMP	SendB3		; 3 Si CARRY=0, sigue en SendB3
Bit3Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB3
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 3 ***
;---------------------------------------------------------------------------------------
CalcB4
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 4 de TXBYTE
	BCS	Bit4Is1		; 2 Si CARRY=1 sigue en Bit4Is1
Bit4Is0
	JMP	SendB4		; 3 Si CARRY=0, sigue en SendB4
Bit4Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB4
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 4 ***
;---------------------------------------------------------------------------------------
CalcB5
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 5 de TXBYTE
	BCS	Bit5Is1		; 2 Si CARRY=1 sigue en Bit5Is1
Bit5Is0
	JMP	SendB5		; 3 Si CARRY=0, sigue en SendB5
Bit5Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB5
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 5 ***
;---------------------------------------------------------------------------------------
CalcB6
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 6 de TXBYTE
	BCS	Bit6Is1		; 2 Si CARRY=1 sigue en Bit6Is1
Bit6Is0
	JMP	SendB6		; 3 Si CARRY=0, sigue en SendB6
Bit6Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB6
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 6 ***
;---------------------------------------------------------------------------------------
CalcB7
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ROR	TXBYTE		; 6 Obtiene en CARRY el bit 7 de TXBYTE
	BCS	Bit7Is1		; 2 Si CARRY=1 sigue en Bit7Is1
Bit7Is0
	JMP	SendB7		; 3 Si CARRY=0, sigue en SendB7
Bit7Is1				; 1 Para llegar aqui, se consumio 1 ciclo extra de reloj
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
SendB7
	STA	$DD00		; 4 Escribe el valor al puerto
				; *** Comienzo del bit 7 ***
;---------------------------------------------------------------------------------------
CalcStop
	TXA			; 2 Recupera en A el valor del puerto (almacenado en X)
	ORA	#$04		; 2 Al contenido de A se le coloca el bit 2 (TX) a 1
	NOP			; 2
	NOP			; 2
	NOP			; 2
	NOP			; 2
	NOP			; 2
SendStop
	STA	$DD00		; 4 *** Comienzo del bit de stop ***
;---------------------------------------------------------------------------------------
	NOP			; 2
	NOP			; 2
	NOP			; 2
	NOP			; 2
	NOP			; 2
	NOP			; 2
	RTS			; 2
	NOP			; 2
	NOP			; 2
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Subrutina ReadByte, recibe un byte y lo almacena en RXBYTE

ReadByte
	LDA	#$00		; 2 Coloca 85 ($0055) microsegundos en el Timer A
	STA	$DD05		; 4
	LDA	#$55		; 2
	STA	$DD04		; 4
	LDA	$DD0D		; 4 Limpia los bits de interrupcion (CIA2: NMI)
	LDA	#$09		; 2 Arranca el Timer A
	STA	$DD0E		; 4
	LDA	#$90		; 2 Habilita la interrupcion (NMI) por FLAG2
	STA	$DD0D		; 4
;	LDA	#$06		; 2 Pone en 1 la linea RTS (y en 1 la linea DTR, para habilitar al modulo wifi)
	LDA	#$02		; 2 Pone en 1 la linea RTS (y en 0 la linea DTR, para habilitar al sintetizador)
	STA	$DD01		; 4
WaitStrt
	LDA	$DD0D		; 4 Si no transcurrieron los 85 microsegundos del Timer A
	AND	#$01		; 2
	BEQ	WaitStrt	; 2 vuelve a WaitStrt a esperar el bit de start
	JMP	CancelRX	; 3 Si transcurrio el tiempo, entonces pasaron 85 us sin recepcion, sigue en CancelRX
StartBit			; 14 Para llegar aqui, se consumieron 14 ciclos de reloj
	LDA	#$10		; 2 Deshabilita la interrupcion (NMI) por FLAG2
	STA	$DD0D		; 4
ReadB0
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	NOP			; 2
	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
ReadB1
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	NOP			; 2
	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
ReadB2
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
;	LDA	#$04		; 2 Pone en 0 la linea RTS (y mantiene en 1 la linea DTR, para habilitar al modulo wifi)
	LDA	#$00		; 2 Pone en 0 la linea RTS (y mantiene en 0 la linea DTR, para habilitar al sintetizador)
	STA	$DD01		; 4
ReadB3
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	NOP			; 2
	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
ReadB4
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	NOP			; 2
	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
ReadB5
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	LDA	$DD0D		; 4 Limpia los bits de interrupcion (CIA2: NMI)
	NOP			; 2
;	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
;	NOP			; 2
;	NOP			; 2
ReadB6
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	NOP			; 2
	JMP	*+3		; 3 Salta a la instruccion siguiente, para perder 3 ciclos
ReadB7
	LDA	$DD01		; 4 Lee en A el valor del puerto B
	ROR			; 2 Transfiere el bit 0 (RX) a CARRY
	ROR	RXBYTE		; 6 Ingresa CARRY por el MSB de RXBYTE
	PLA			; 4 Elimina de la pila la llamada a NMI
	PLA			; 4
	PLA			; 4
	RTS			; 6 Retorna
CancelRX
	LDA	#$10		; 2 Deshabilita la interrupcion (NMI) por FLAG2
	STA	$DD0D		; 4
;	LDA	#$04		; 2 Pone en 0 la linea RTS (y mantiene en 1 la linea DTR, para habilitar al modulo wifi)
	LDA	#$00		; 2 Pone en 0 la linea RTS (y mantiene en 0 la linea DTR, para habilitar al sintetizador)
	STA	$DD01		; 4
	LDA	#$00		; 2 Carga RXBYTE con el valor 0
	STA	RXBYTE		; 4
	RTS			; 6 Retorna

;///////////////////////////////////////////////////////////////////////////////////
; Subrutina ChkVersion, consulta el modelo y version del sintetizador

ChkVersion
	SEI			; Deshabilita las interrupciones
	LDA	#%01111111
	STA	$DC0D		; Deshabilita las interrupciones desde CIA 1
	AND	$D011
	STA	$D011		; Pone a 0 el bit mas significativo del registro de barrido (raster) del VIC
	LDA	$D011		; Deshabilita el VIC II (apaga la pantalla)
	AND	#$EF
	STA	$D011
Border
	LDA	$D011		; Espera a que el video llegue al borde inferior de la pantalla (bit 8 = 1, linea>255)
	AND	#%10000000
	BEQ	Border
	LDA	#<StartBit	; Apunta el vector de NMI a StartBit
	STA	$0318
	LDA	#>StartBit
	STA	$0319
	LDA	#6		; Coloca PB1 (RTS) y PB2 (DTR) como salida, y el resto como entrada en el puerto del usuario
	STA	$DD03
	LDA	$DD02		; Coloca PA2 (TXD) como salida en el puerto del usuario
	ORA	#4
	STA	$DD02
	LDA	$DD00		; Coloca un 1 en PA2 (TX: estado de reposo)
	ORA	#$04
	STA	$DD00

	LDA	#27		; Envia el caracter ESC
	JSR	SendByte
	JSR	ReadByte	; Recibe la respuesta (si no esta activado el eco, no habra respuesta)
	LDA	#"V"		; Envia el caracter "V"
	JSR	SendByte
	JSR	ReadByte	; Recibe la respuesta, si no es "H", sigue en ChkVErr
	LDA	RXBYTE
	CMP	#$48
	BNE	ChkVErr
	STA	VERSION		; Escribe "H" en VERSION
;	JSR	CHROUT
	LDA	#0		; Envia el caracter NUL
	JSR	SendByte
	JSR	ReadByte
	LDA	RXBYTE		; Escribe la respuesta en VERSION+1 ("R"|"D")
	STA	VERSION+1
;	JSR	CHROUT
	LDA	#0		; Envia el caracter NUL
	JSR	SendByte
	JSR	ReadByte
	LDA	RXBYTE		; Escribe la respuesta en VERSION+2 ("4"|"6"|"1")
	STA	VERSION+2
;	TAX
;	LDA	#$00
;	JSR	$BDCD
	LDA	#0		; Envia el caracter NUL
	JSR	SendByte
	JSR	ReadByte
	LDA	RXBYTE		; Escribe la respuesta en VERSION+3 (version 1)
	STA	VERSION+3
;	TAX
;	LDA	#$00
;	JSR	$BDCD
	LDA	#0		; Envia el caracter NUL
	JSR	SendByte
	JSR	ReadByte
	LDA	RXBYTE		; Escribe la respuesta en VERSION+4 (version 2)
	STA	VERSION+4
;	TAX
;	LDA	#$00
;	JSR	$BDCD
	LDA	#0		; Envia el caracter NUL
	JSR	SendByte
	JSR	ReadByte
	LDA	RXBYTE		; Escribe la respuesta en VERSION+5 (version 3)
	STA	VERSION+5
;	TAX
;	LDA	#$00
;	JSR	$BDCD
ChkVEnd
	LDA	#$47		; Apunta el vector de NMI a la rutina por defecto de la ROM
	STA	$0318
	LDA	#$FE
	STA	$0319
	LDA	$D011		; Habilita de nuevo el VIC II
	ORA	#$10
	STA	$D011
	LDA	#%10000001
	STA	$DC0D		; Habilita interrupcion por timer A de CIA 1
	CLI			; Vuelve a habilitar las interrupciones
	RTS			; Retorna
ChkVErr
	LDA	#$00		; Escribe 0 en VERSION (no se detecto el sintetizador)
	STA	VERSION
	JMP	ChkVEnd		; y sigue en ChkVEnd

;///////////////////////////////////////////////////////////////////////////////////
; Inicializa interrupciones
;///////////////////////////////////////////////////////////////////////////////////

InitIrq
	SEI			; Deshabilita las interrupciones
	LDA	#%01111111
	STA	$DC0D		; Deshabilita las interrupciones desde CIA 1
	AND	$D011
	STA	$D011		; Pone a 0 el bit mas significativo del registro de barrido (raster) del VIC
	LDA	#251
	STA	$D012		; Selecciona el numero de linea donde deberia dispararse la interrupcion del VIC
	LDA	#<Irq
	STA	$0314
	LDA	#>Irq
	STA	$0315		; Ajusta el vector de interrupcion para apuntar a la rutina Irq
	LDA	#$00
	STA	TXINDEXOUT
	STA	TXINDEXIN
	STA	TXBUFFERCNT
	STA	PRINDEXOUT
	STA	PRINDEXIN
	STA	PRBUFFERCNT
	LDA	#%00000001
	STA	$D01A		; Habilita interrupcion de barrido del VIC
	LDA	$DC0D		; Limpia los bits de interrupcion (CIA1: IRQ)
	LDA	$DD0D		; (CIA2: NMI)
	LDA	#$06		; Coloca PB1 (RTS) y PB2 (DTR) como salidas
	STA	$DD03
	LDA	#$00		; Pone en 0 las lineas RTS y DTR, para habilitar al sintetizador
	STA	$DD01
	LDA	$DD00		; Coloca un 1 en PA2 (TX: estado de reposo)
	ORA	#$04
	STA	$DD00
	LDA	#$00		; Inicializa el SID
	STA	SIDREG+24	; Volumen = 0
	LDA	#$00		; Attack/decay = 0 en los 3 canales
	STA	SIDREG+5
	STA	SIDREG+12
	STA	SIDREG+19
	LDA	#$F0		; Sustain = 15 / Release = 0 en el canal 1 (beeps)
;	LDA	#$00		; Sustain = 0 / Release = 0 en el canal 1 (beeps)
	STA	SIDREG+6
	LDA	#$00		; Sustain = 0 / Release = 0 en los canales 2 (click) y 3 (no usado)
	STA	SIDREG+13
	STA	SIDREG+20
	LDA	#$10		; Onda triangular en canal 1 / sonido apagado
	STA	SIDREG+4
	LDA	#$80		; Ruido en canal 2 / sonido apagado
	STA	SIDREG+11
	LDA	#$40		; Pulso en canal 3 (no usado) / sonido apagado
	STA	SIDREG+18
	LDA	#$0F		; Frecuencia canal 2 (click) = 229,758 Hz
	STA	SIDREG+7
	STA	SIDREG+8
	LDA	#$0F		; Volumen = 15
	STA	SIDREG+24
	CLI			; Vuelve a habilitar las interrupciones
	RTS			; Fin de la inicializacion, se retorna al BASIC

;///////////////////////////////////////////////////////////////////////////////////
; Restaura interrupciones
;///////////////////////////////////////////////////////////////////////////////////

RestoreIrq
	SEI			; Deshabilita las interrupciones
	LDA	#%10000001
	STA	$DC0D		; Habilita interrupcion por timer A de CIA 1
	LDA	#$31
	STA	$0314
	LDA	#$EA
	STA	$0315		; Ajusta el vector de interrupcion a los valores por defecto
	LDA	$DC0D		; Limpia los bits de interrupcion (CIA1: IRQ)
	LDA	$DD0D		; (CIA2: NMI)
	LDA	#%00000000
	STA	$D01A		; Deshabilita interrupcion de barrido del VIC
	CLI			; Vuelve a habilitar las interrupciones
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Rutina de interrupcion
;///////////////////////////////////////////////////////////////////////////////////

Irq
	SEI			; Deshabilita las interrupciones
	PHA			; Coloca el registro A en la pila
	TXA
	PHA			; Coloca el registro X en la pila

	LDA	#$FF		; this is the orthodox and safe way of clearing the interrupt condition of the VICII.
	STA	$D019		; if you don't do this the interrupt condition will be present all the time and you end
				; up having the CPU running the interrupt code all the time, as when it exists the
				; interrupt, the interrupt request from the VICII will be there again regardless of the
				; rasterline counter.

	LDA	#%00000000
	STA	$D01A		; Deshabilita interrupcion de barrido del VIC
	LDA	#$80		; Ruido en canal 2 / sonido apagado
	STA	SIDREG+11
TXSpeech
	LDA	TXBUFFERCNT	; Si no hay elementos en el buffer para enviar, sigue en PRText
	BEQ	PRText
;	LDA	#13
;	STA	$D020		; Color de borde: verde claro
	LDA	$DD01
	AND	#%01000000
	BEQ	PRText
	LDX	TXINDEXOUT
	LDA	Speech, X
	JSR	SendByte
	INC	TXINDEXOUT
	DEC	TXBUFFERCNT
PRText
	LDA	CONTROLBYTE	; Si el bit 0 de CONTROLBYTE es 1
	AND	#$01
	BEQ	PRText1
	LDA	#$00		; Habilita el parpadeo del cursor
	STA	$CC
PRText1	LDA	PRBUFFERCNT	; Si no hay elementos en el buffer para imprimir, sigue en MuteSID
	BEQ	MuteSID
;	LDA	#2
;	STA	$D020		; Color de borde: rojo
	LDA	PRBUFFERCNT	; Si PRBUFFERCNT es impar sigue en PRText1a
	AND	#$01
	BNE	PRText1a
	LDA	#$A4		; Frecuencia canal 1 = 671 Hz
	STA	SIDREG
	LDA	#$3F
	STA	SIDREG+1
	LDA	#$11		; Onda triangular en canal 1 / sonido activado
	STA	SIDREG+4
	JMP	PRText1b	; Sigue en PRText1b
PRText1a
	LDA	#$E5		; Frecuencia canal 1 = 1219 Hz
	STA	SIDREG
	LDA	#$4F
	STA	SIDREG+1
	LDA	#$11		; Onda triangular en canal 1 / sonido activado
	STA	SIDREG+4
	LDA	#$81		; Ruido en canal 2 / sonido activado
	STA	SIDREG+11
PRText1b
	LDX	PRINDEXOUT
;	SEI			; Deshabilita las interrupciones
;	LDA	#%00000000
;	STA	$D01A		; Deshabilita interrupcion de barrido del VIC
	LDA	Buffer, X	; Si el caracter a imprimir no es RETURN, sigue en PRText2
	CMP	#$0D
	BNE	PRText2
	LDA	#$20		; Si es RETURN, imprime antes un espacio
	JSR	CHROUT
PRText2	LDA	Buffer, X
	JSR	CHROUT
	LDA	#$01
	STA	$CD
	LDA	#$00
	STA	$CF
	INC	PRINDEXOUT
	DEC	PRBUFFERCNT

;Pause	LDX	#90
;	DEX
;	INC	$D020
;	BNE	Pause		; Bucle vacio que no hace nada por alrededor de medio milisegundo
ExitIrq
;	LDA	#0
;	STA	$D020		; Color de borde: negro
	ASL	$D019		; Reconoce la interrupcion poniendo a 0 la bandera de interrupcion del VIC
	LDA	#%00000001
	STA	$D01A		; Habilita interrupcion de barrido del VIC
;	CLI			; Vuelve a habilitar las interrupciones

	PLA
	TAX			; Restaura el registro X desde la pila
	PLA			; Restaura el registro A desde la pila
	JMP	$EA31		; Salta a la rutina de interrupcion estandar de la ROM para leer el teclado, procesar el cursor, etc.

MuteSID
	LDA	#$10		; Onda triangular en canal 1 / sonido apagado
	STA	SIDREG+4
;	LDA	#$80		; Ruido en canal 2 / sonido apagado
;	STA	SIDREG+11
	JMP	ExitIrq		; Sigue en ExitIrq

;///////////////////////////////////////////////////////////////////////////////////
; Carga en el buffer de transmision el contenido de la variable ZV$
;///////////////////////////////////////////////////////////////////////////////////

SendText
	LDA	VARTAB		; Apunta VARPTR al comienzo de la tabla de variables simples
	STA	VARPTR
	LDA	VARTAB+1
	STA	VARPTR+1
ChkEnd
	LDA	VARPTR		; Verifica si VARPTR llego hasta la direccion apuntada
	CMP	ARYTAB		; por ARYTAB
	BNE	ChkVar
	LDA	VARPTR+1
	CMP	ARYTAB+1
	BNE	ChkVar
	RTS
ChkVar
	LDX	#$00		; Inicializa X a 0
	LDA	VARPTR		; Apunta ChkVar1 y ChkVar2 a la direccion apuntada por
	STA	ChkVar1+1	; VARPTR
	STA	ChkVar2+1
	LDA	VARPTR+1
	STA	ChkVar1+2
	STA	ChkVar2+2
ChkVar1
	LDA	VARPTR, X	; Verifica si el primer caracter del nombre de la variable
	CMP	#"Z"		; es Z y tiene el bit 7 desactivado
	BNE	NextVar
	INX
ChkVar2
	LDA	VARPTR, X	; Verifica si el segundo caracter del nombre de la variable
	CMP	#"V"+128	; es V y tiene el bit 7 activado
	BEQ	ReadVar		; Si se encontro la variable, sigue en ReadVar
NextVar
	LDA	#7		; Apunta VARPTR a la siguiente variable (VARPTR = VARPTR + 7)
	JSR	IncVPtr
	JMP	ChkEnd		; y vuelve a ChkEnd
ReadVar
	LDA	#2		; Apunta VARPTR a la longitud de la variable (VARPTR = VARPTR + 2)
	JSR	IncVPtr
	LDA	VARPTR		; Apunta ReadVar1 a la direccion apuntada por VARPTR
	STA	ReadVar1+1
	LDA	VARPTR+1
	STA	ReadVar1+2
ReadVar1
	LDA	Buffer		; Coloca la longitud de la variable en CHARCNT
	STA	CHARCNT
	LDA	#1		; Apunta VARPTR al puntero al texto de la variable (VARPTR = VARPTR + 1)
	JSR	IncVPtr
	LDA	VARPTR
	STA	ReadVar2+1
	STA	ReadVar3+1
	LDA	VARPTR+1
	STA	ReadVar2+2
	STA	ReadVar3+2
ReadVar2
	LDA	Buffer
	STA	CopyLoop1+1
	LDX	#$01
ReadVar3
	LDA	Buffer, X
	STA	CopyLoop1+2

	LDX	#$00		; Inicializa X a 0
CopyLoop
	LDA	CHARCNT		; Si todavia quedan caracteres para copiar, sigue en CopyLoop1
	BNE	CopyLoop1
	RTS			; sino retorna
CopyLoop1
	LDA	Buffer, X	; Lee un byte de la variable
	JSR	AddToTXBuffer	; y llama a AddToTXBuffer para agregarlo al buffer de transmision
	INX			; Pasa al siguiente caracter
	DEC	CHARCNT
	JMP	CopyLoop	; y vuelve a CopyLoop
	
;	TAX
;	LDA	#$00
;	JSR	$BDCD
;	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Carga en el buffer de impresion el contenido de la variable ZP$
;///////////////////////////////////////////////////////////////////////////////////

PrintText
	LDA	VARTAB		; Apunta VARPTR al comienzo de la tabla de variables simples
	STA	VARPTR
	LDA	VARTAB+1
	STA	VARPTR+1
PChkEnd
	LDA	VARPTR		; Verifica si VARPTR llego hasta la direccion apuntada
	CMP	ARYTAB		; por ARYTAB
	BNE	PChkVar
	LDA	VARPTR+1
	CMP	ARYTAB+1
	BNE	PChkVar
	RTS
PChkVar
	LDX	#$00		; Inicializa X a 0
	LDA	VARPTR		; Apunta PChkVar1 y PChkVar2 a la direccion apuntada por
	STA	PChkVar1+1	; VARPTR
	STA	PChkVar2+1
	LDA	VARPTR+1
	STA	PChkVar1+2
	STA	PChkVar2+2
PChkVar1
	LDA	VARPTR, X	; Verifica si el primer caracter del nombre de la variable
	CMP	#"Z"		; es Z y tiene el bit 7 desactivado
	BNE	PNextVar
	INX
PChkVar2
	LDA	VARPTR, X	; Verifica si el segundo caracter del nombre de la variable
	CMP	#"P"+128	; es P y tiene el bit 7 activado
	BEQ	PReadVar	; Si se encontro la variable, sigue en PReadVar
PNextVar
	LDA	#7		; Apunta VARPTR a la siguiente variable (VARPTR = VARPTR + 7)
	JSR	IncVPtr
	JMP	PChkEnd		; y vuelve a PChkEnd
PReadVar
	LDA	#2		; Apunta VARPTR a la longitud de la variable (VARPTR = VARPTR + 2)
	JSR	IncVPtr
	LDA	VARPTR		; Apunta PReadVar1 a la direccion apuntada por VARPTR
	STA	PReadVar1+1
	LDA	VARPTR+1
	STA	PReadVar1+2
PReadVar1
	LDA	Buffer		; Coloca la longitud de la variable en CHARCNT
	STA	CHARCNT
	LDA	#1		; Apunta VARPTR al puntero al texto de la variable (VARPTR = VARPTR + 1)
	JSR	IncVPtr
	LDA	VARPTR
	STA	PReadVar2+1
	STA	PReadVar3+1
	LDA	VARPTR+1
	STA	PReadVar2+2
	STA	PReadVar3+2
PReadVar2
	LDA	Buffer
	STA	PCopyLoop1+1
	LDX	#$01
PReadVar3
	LDA	Buffer, X
	STA	PCopyLoop1+2

	LDX	#$00		; Inicializa X a 0
PCopyLoop
	LDA	CHARCNT		; Si todavia quedan caracteres para copiar, sigue en PCopyLoop1
	BNE	PCopyLoop1
	RTS			; sino retorna
PCopyLoop1
	LDA	Buffer, X	; Lee un byte de la variable
	JSR	AddToPRBuffer	; y llama a AddToPRBuffer para agregarlo al buffer de transmision
	INX			; Pasa al siguiente caracter
	DEC	CHARCNT
	JMP	PCopyLoop	; y vuelve a PCopyLoop
	
;///////////////////////////////////////////////////////////////////////////////////
; Rutina auxiliar para incrementar el puntero VARPTR

IncVPtr
	CLC			; Incrementa VARPTR en la cantidad cargada en A
	ADC	VARPTR
	STA	VARPTR
	BCC	IVPEnd
	INC	VARPTR+1
IVPEnd
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Agrega el caracter cargado en A, al buffer de transmision
;///////////////////////////////////////////////////////////////////////////////////

AddToTXBuffer
	STA	TXBYTE		; Guarda el byte en TXBYTE
	LDY	TXINDEXIN	; Carga en Y el valor de TXINDEXIN
ChkBuffer
	LDA	TXBUFFERCNT	; Si TXBUFFERCNT=255 (buffer lleno) espera hasta que se libere al menos 1 byte
	EOR	#$FF
	BEQ	ChkBuffer
	SEI			; Si hay lugar en el buffer, deshabilita las interrupciones
	LDA	TXBYTE		; coloca TXBYTE en el siguiente byte libre del buffer (Speech)
	STA	Speech, Y
	INC	TXINDEXIN	; Incrementa el indice TXINDEXIN
	INC	TXBUFFERCNT	; y el contador TXBUFFERCNT
	CLI			; Vuelve a habilitar las interrupciones
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Agrega el caracter cargado en A, al buffer de transmision
;///////////////////////////////////////////////////////////////////////////////////

AddToPRBuffer
	STA	PRBYTE		; Guarda el byte en PRBYTE
	LDY	PRINDEXIN	; Carga en Y el valor de PRINDEXIN
ChkBuffer_
	LDA	PRBUFFERCNT	; Si PRBUFFERCNT=255 (buffer lleno) espera hasta que se libere al menos 1 byte
	EOR	#$FF
	BEQ	ChkBuffer_
	SEI			; Si hay lugar en el buffer, deshabilita las interrupciones
	LDA	PRBYTE		; coloca PRBYTE en el siguiente byte libre del buffer (Buffer)
	STA	Buffer, Y
	INC	PRINDEXIN	; Incrementa el indice PRINDEXIN
	INC	PRBUFFERCNT	; y el contador PRBUFFERCNT
	CLI			; Vuelve a habilitar las interrupciones
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Espera a que se haya enviado completo el buffer de transmision	49158
;///////////////////////////////////////////////////////////////////////////////////

WaitSend
	LDA	TXBUFFERCNT	; Espera hasta que TXBUFFERCNT sea 0
	BNE	WaitSend
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Espera a que se haya impreso completo el buffer de impresion		49161
;///////////////////////////////////////////////////////////////////////////////////

WaitPrint
	LDA	PRBUFFERCNT	; Espera hasta que PRBUFFERCNT sea 0
	BNE	WaitPrint
	RTS

;///////////////////////////////////////////////////////////////////////////////////
; Buffers (pantalla+RS232)
;///////////////////////////////////////////////////////////////////////////////////

Buffer

	*= Buffer+256
Speech
